""" Dash Server """

#import gevent.monkey
#gevent.monkey.patch_all()

import dash
import dash_bootstrap_components as dbc
from dash import dcc
from dash import html
from dash.dependencies import Output
from dash.dependencies import Input
from dash.dependencies import State
from dash.dependencies import ALL
from dash.dependencies import MATCH
from dash.dependencies import ClientsideFunction
from dash.exceptions import PreventUpdate
import flask
from flask import g, make_response, request
from flask_caching import Cache
#from pyinstrument import Profiler
from pathlib import Path
from datetime import datetime as dt
from datetime import timedelta
from urllib.parse import urlparse, parse_qs

from data_handler import DEFAULT_VAR
from data_handler import DEFAULT_MODEL
from data_handler import VARS 
from data_handler import MODELS
from data_handler import DEBUG
from data_handler import DATES
from data_handler import cache, cache_timeout 
from data_handler import ALIASES
from data_handler import ROUTE_DEFAULTS

from tabs.forecast import tab_forecast
from tabs.forecast import sidebar_forecast
from tabs.evaluation import tab_evaluation
from tabs.evaluation import sidebar_evaluation
from tabs.observations import tab_observations
from tabs.observations import sidebar_observations
from tabs.fullscreen import go_fullscreen 

import uuid
import socket

HOSTNAME = socket.gethostbyname_ex(socket.gethostname())[0]

if HOSTNAME in ('bscesdust03.bsc.es', 'dust.hqads.aemet.es'):
    pathname = '/daily_dashboard/'
else:
    pathname = '/dashboard/'

TIMEOUT = 10

fontawesome = 'https://cdnjs.cloudflare.com/ajax/libs/font-awesome/5.15.4/css/all.min.css'
leaflet = "https://cdnjs.cloudflare.com/ajax/libs/leaflet/1.2.0/leaflet.css"

srv = flask.Flask(__name__)
app = dash.Dash(__name__,
                external_scripts=['https://code.jquery.com/jquery-3.6.0.slim.min.js'],
                external_stylesheets=[dbc.themes.BOOTSTRAP,
                                      dbc.themes.GRID,
                                      fontawesome,
                                      leaflet
                                     ],
                url_base_pathname=pathname,
                server=srv,
                prevent_initial_callbacks=True
                )
app.css.config.serve_locally = True
app.scripts.config.serve_locally = True
app.config.suppress_callback_exceptions = True
server = app.server

cache.init_app(server)

try:
    cache.clear()
except Exception as e:
    print('CACHE CLEAR ERROR:', str(e))
    pass

app.index_string = """
<!DOCTYPE html>
<html>
    <head>
        <script>
            L_PREFER_CANVAS=true;
            L_DISABLE_3D=true;
            L_SCROLL_WHEEL_ZOOM=false;

            HTMLCanvasElement.prototype.getContext = function(origFn) {
            return function(type, attribs) {
            attribs = attribs || {};
            attribs.preserveDrawingBuffer = true;
            return origFn.call(this, type, attribs);
             };
            }(HTMLCanvasElement.prototype.getContext);
        </script>
        <!-- Global site tag (gtag.js) - Google Analytics -->
        <!--
        <script async src="https://www.googletagmanager.com/gtag/js?id=G-5V3R8V4021"></script>
        <script>
          window.dataLayer = window.dataLayer || [];
          function gtag(){dataLayer.push(arguments);}
          gtag('js', new Date());

          gtag('config', 'G-5V3R8V4021');
        </script>
        -->
        {%metas%}
        <title>{%title%}</title>
        {%favicon%}
        {%css%}
    </head>
    <body>
        {%app_entry%}
        <footer>
            {%config%}
            {%scripts%}
            {%renderer%}
        </footer>
    </body>
</html>
"""
if DEBUG: print('SERVER: start creating app layout')

#SET APP LAYOUT TO BE POPULATED
app.layout = html.Div([
    dcc.Location(id="url", refresh=False)
], id="content")

def get_input_aliases(route_selections):
    """ Change user inputs into internal variables"""
    for key, value in route_selections.items():
        #preserve the list of models by only changing the individual model name
        if key == 'model':
            route_selections[key] = ['median' if x=='multi-model' else x for x in route_selections[key]]
        elif value[0] in ALIASES:
            route_selections[key] = [ALIASES[value[0]]]
    print('-- GET_INPUT_ALIASES ', route_selections)
    return route_selections

def eval_section_query(queries):
    """pull SECTION var from url query and assign value to appropriate option"""
    if 'section' in queries:
        if queries['tab']==['forecast-tab']:
            queries['for_option'] = queries['section']
        elif queries['tab']==['evaluation-tab']:
            queries['eval_option'] = queries['section']
        elif queries['tab']==['observations-tab']:
            queries['obs_option'] = queries['section']
    return queries   

def get_url_queries(url, ROUTE_DEFAULTS=ROUTE_DEFAULTS):
    """ pull all of the keyword arguments from url and update ROUTE_DEFAULTS dict as necessary""" 
    parsed_url = urlparse(url, scheme='http', allow_fragments=False)
    queries = parse_qs(parsed_url[4])
    print('--- URL QUERIES--', queries)
    route_selections = ROUTE_DEFAULTS.copy()
    route_selections.update(queries)
    #now get section and assign to appropriate option
    route_selections = get_input_aliases(route_selections)
    route_selections = eval_section_query(route_selections)
    return route_selections

@app.callback(
    Output('app-sidebar', 'children'),
    [Input('app-tabs', 'value')],
)
def render_sidebar(tab, default_var=DEFAULT_VAR, default_model=[DEFAULT_MODEL], obs_option='rgb', for_option='models'):
    """ Function rendering requested tab """
    tabs = {
        'forecast-tab' : sidebar_forecast(VARS, default_var, MODELS, default_model, window=for_option),
        'evaluation-tab' : sidebar_evaluation(),
        'observations-tab' :  sidebar_observations(obs_option),
        'fullscreen-tab' : 'Full'
    }
    if tabs[tab][0] is 'Full':
        return 
    return tabs[tab]

@app.callback(
    Output("content", "children"),
    Input("url", "href")
)
def router(url):
    """ Get url search queries and build layout for app"""
    route_selections = get_url_queries(url)
    try:
        children = [
            html.Div(
                id='app-sidebar',
                children=render_sidebar(tab=route_selections['tab'][0],
                    default_var=route_selections['var'][0],
                    default_model=route_selections['model'],
                    obs_option=route_selections['obs_option'][0],
                    for_option=route_selections['for_option'][0]
                    ), 
                className='sidebar'
            ),
            dcc.Tabs(id='app-tabs', value=route_selections['tab'][0] , children=[
                tab_forecast(window=route_selections['for_option'][0], end_date=route_selections['date'][0]),#'models', 'prob', 'was'
                tab_evaluation(route_selections['eval_option'][0]), #nrt or scores
                tab_observations(route_selections['obs_option'][0]),#rgb or visibility
                # go_fullscreen(pathname),
            ]),
        ]
    except: #This handles when user inputs incorrect URL params
        children = [
            html.Div(
                id='app-sidebar',
                children=render_sidebar(tab='forecast-tab' ), 
                className='sidebar'
            ),
            dcc.Tabs(id='app-tabs', value='forecast-tab' , children=[
                tab_forecast(),
                tab_evaluation(),
                tab_observations(),
                # go_fullscreen(pathname),
            ]),
        ]
    return children 

if DEBUG: print('SERVER: stop creating app layout')

from tabs.forecast_callbacks import *
from tabs.evaluation_callbacks import *
from tabs.observations_callbacks import *

if __name__ == '__main__':
    app.run_server(debug=True, # use_reloader=False,  # processes=4, threaded=False,
                   host='0.0.0.0', port=7778)
